"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = callbackHandler;

var _errors = require("../../lib/errors");

var _utils = require("./utils");

async function callbackHandler(sessionToken, profile, account, options) {
  if (!(account !== null && account !== void 0 && account.id) || !account.type) throw new Error("Missing or invalid provider account");
  if (!["email", "oauth"].includes(account.type)) throw new Error("Provider not supported");
  const {
    adapter,
    jwt,
    events,
    session: {
      jwt: useJwtSession
    }
  } = options;

  if (!adapter) {
    return {
      user: profile,
      account,
      session: {}
    };
  }

  const {
    createUser,
    updateUser,
    getUser,
    getUserByProviderAccountId,
    getUserByEmail,
    linkAccount,
    createSession,
    getSession,
    deleteSession
  } = adapter;
  let session = null;
  let user = null;
  let isNewUser = false;

  if (sessionToken) {
    var _session2;

    if (useJwtSession) {
      try {
        var _session;

        session = await jwt.decode({ ...jwt,
          token: sessionToken
        });

        if ((_session = session) !== null && _session !== void 0 && _session.sub) {
          user = await getUser(session.sub);
        }
      } catch (_unused) {}
    }

    session = await getSession(sessionToken);

    if ((_session2 = session) !== null && _session2 !== void 0 && _session2.userId) {
      user = await getUser(session.userId);
      isSignedIn = !!user;
    }
  }

  if (account.type === "email") {
    const userByEmail = profile.email && (await getUserByEmail(profile.email));

    if (userByEmail) {
      var _user, _events$updateUser;

      if (((_user = user) === null || _user === void 0 ? void 0 : _user.id) !== userByEmail.id && !useJwtSession && sessionToken) {
        await deleteSession(sessionToken);
      }

      const currentDate = new Date();
      user = await updateUser({ ...userByEmail,
        emailVerified: currentDate
      });
      await ((_events$updateUser = events.updateUser) === null || _events$updateUser === void 0 ? void 0 : _events$updateUser.call(events, {
        user
      }));
    } else {
      var _events$createUser;

      const currentDate = new Date();
      user = await createUser({ ...profile,
        emailVerified: currentDate
      });
      await ((_events$createUser = events.createUser) === null || _events$createUser === void 0 ? void 0 : _events$createUser.call(events, {
        user
      }));
      isNewUser = true;
    }

    session = useJwtSession ? {} : await createSession({
      userId: user.id,
      expires: (0, _utils.fromDate)(options.session.maxAge)
    });
    return {
      session,
      user,
      isNewUser
    };
  } else if (account.type === "oauth") {
    const userByProviderAccountId = await getUserByProviderAccountId(account.provider, account.id);

    if (userByProviderAccountId) {
      if (user) {
        if (userByProviderAccountId.id === user.id) {
          return {
            session,
            user,
            isNewUser
          };
        }

        throw new _errors.AccountNotLinkedError();
      }

      session = useJwtSession ? {} : await createSession({
        userId: userByProviderAccountId.id,
        expires: (0, _utils.fromDate)(options.session.maxAge)
      });
      return {
        session,
        user: userByProviderAccountId,
        isNewUser
      };
    } else {
      var _events$createUser2, _events$linkAccount2;

      if (user) {
        var _events$linkAccount;

        await linkAccount(user.id, account);
        await ((_events$linkAccount = events.linkAccount) === null || _events$linkAccount === void 0 ? void 0 : _events$linkAccount.call(events, {
          user,
          account
        }));
        return {
          session,
          user,
          isNewUser
        };
      }

      const userByEmail = profile.email && (await getUserByEmail(profile.email));

      if (userByEmail) {
        throw new _errors.AccountNotLinkedError();
      }

      user = await createUser({ ...profile,
        emailVerified: null
      });
      await ((_events$createUser2 = events.createUser) === null || _events$createUser2 === void 0 ? void 0 : _events$createUser2.call(events, {
        user
      }));
      await linkAccount(user.id, account);
      await ((_events$linkAccount2 = events.linkAccount) === null || _events$linkAccount2 === void 0 ? void 0 : _events$linkAccount2.call(events, {
        user,
        account
      }));
      session = useJwtSession ? {} : await createSession({
        userId: user.id,
        expires: (0, _utils.fromDate)(options.session.maxAge)
      });
      return {
        session,
        user,
        isNewUser: true
      };
    }
  }
}